from . import mdundo_url
import requests
from bs4 import BeautifulSoup


class Genre:

    def __init__(self):
        self.genre_url =  f"{mdundo_url}/best"
        self.url = f"{mdundo_url}/genres"

    def get_genres(self):
        res = requests.get(self.genre_url)
        soup = BeautifulSoup(res.text, 'lxml')
        genre_wrapper = soup.find("div",{"class":"heading_select"})
        genre_options = genre_wrapper.find_all("option")
        results = []
        for genre in genre_options:
            value = genre["value"]
            genre_id = value.split("/")[-1]
            if genre_id.isnumeric():
                item = {}
                item["id"] = genre_id
                item["name"] = genre.text
                results.append(item)
        return {"result":results,"count":len(results)}

    def get_genre_top_songs(self, genre_id):
        if not genre_id or not genre_id.isnumeric():
            raise ValueError("Invalid Id")
        res = requests.get(f"{self.url}/{genre_id}")
        soup = BeautifulSoup(res.text, 'lxml')
        songs_wrapper = soup.find_all("div",{"class":"song-list_wrapper"})
        if len(songs_wrapper) > 0:
            genre_songs = songs_wrapper[0].find_all("li")
            results = []
            for song in genre_songs:
                item = {}
                item["download_url"] = song.find("a",class_ = "btn btn-download")["href"]
                item["artist"] = song.find("span", class_="song-author").text.split(".",1)[1].strip()
                item["genre"] = song.find("div",class_= "song-genre").text.split("/")[0].replace("#","").strip()
                item["title"] = song.find("span",class_="song-title").text
                item["id"] = item["download_url"].split("/")[-1]
                item["source"] = song.find("a",class_ = "btn btn-play stream_play")["alt"]
                results.append(item)
            return {"result":results,"count":len(results)}
        return []



import unittest
from mdundo.charts import Chart
from string import ascii_lowercase
from mdundo import mdundo_url

class ChartTest(unittest.TestCase):

    def test_weekly_to_100(self):
        chart = Chart("ke")
        self.assertEqual(len(chart.weekly_top_100()["result"]),100)
        self.assertEqual(chart.weekly_top_100()["count"],100)


    def test_monthly_top_20(self):
        chart = Chart("ke")
        self.assertEqual(len(chart.monthly_top_20()["result"]),20)
        self.assertEqual(chart.monthly_top_20()["count"],20)

    def test_new_releases(self):
        chart = Chart("ke")
        self.assertNotEqual(chart.new_releases()["result"],[])


import unittest
from mdundo.artist import Artist
from string import ascii_lowercase
from mdundo import mdundo_url

class ArtistTest(unittest.TestCase):

    def test_get_artist(self):
        for c in ascii_lowercase:
            with self.subTest(i=c):
                artist = Artist(c)
                self.assertEqual(artist.artist_url, f"{mdundo_url}/artists/all/{c}")
                self.assertNotEqual(artist.get_artist(), [])
                self.assertGreater(len(artist.get_artist()), 0)

    def test_invalid_get_artist(self):
        artist = Artist("Kdfdfdfdfd")
        self.assertEqual(artist.get_artist(), [])
        self.assertEqual(len(artist.get_artist()), 0)

    def test_get_artist_song(self):
        artist = Artist()
        test_artist = artist.get_artist().get("result")[0]
        self.assertNotEqual(artist.get_artist_song(test_artist["id"]), [])
        self.assertGreater(len(artist.get_artist_song(test_artist["id"])), 0)

    def test_invalid_get_artist_song(self):
        artist = Artist()
        with self.assertRaises(ValueError):
            artist.get_artist_song("h")


import unittest
from mdundo.genre import Genre
from string import ascii_lowercase
from mdundo import mdundo_url


class GenreTest(unittest.TestCase):

    def setUp(self):
        self.genre = Genre()

    def test_get_genres(self):
        self.assertNotEqual(self.genre.get_genres(),[])

    def test_get_genre_top_songs(self):
        test_genre = self.genre.get_genres()["result"][0]
        self.assertNotEqual(self.genre.get_genre_top_songs(test_genre.get("id")),[])

    def test_invalid_get_genre_top_songs(self):
        with self.assertRaises(ValueError):
            self.genre.get_genre_top_songs("h")


from . import mdundo_url
import requests
from bs4 import BeautifulSoup


class Artist:

    def __init__(self,letter=None):
        self.url = f"{mdundo_url}/a/"
        self.artist_url = f"{mdundo_url}/artists/all/{letter.lower()}" if letter else f"{mdundo_url}/artists"

    def get_artist(self):
        res = requests.get(self.artist_url)
        soup = BeautifulSoup(res.text, 'lxml')
        artist_wrapper = soup.find_all("div",{"class":"artist_wrapper"})
        if len(artist_wrapper) > 0:
            artist_list = artist_wrapper[0].find_all("li")
            results = []
            for artist in artist_list:
                item = {}
                item["name"] = artist.find("div",{"class" : "artist_name"}).text.strip()
                item["id"] = artist.find("div",{"class" : "artist_name"}).find("a")["href"].split("/")[-1]
                item["image"] = artist.find("img")["data-src"]
                results.append(item)
            return {"result":results,"count":len(results)}
        return []


    def get_artist_song(self, artist_id):
        if not artist_id or not artist_id.isnumeric():
            raise ValueError("Invalid artis id")
        res = requests.get(f"{self.url}{artist_id}")
        soup = BeautifulSoup(res.text, 'lxml')
        
        song_list_wrapper = soup.find_all("ul",{"id":"song_list"})
        songs_list_wrapper_hidden = None
        if len(song_list_wrapper) > 1:
            songs_list_wrapper_hidden = song_list_wrapper[1]
        if song_list_wrapper:
            artist_image = soup.find("div",{"class":"b-description__top"}).find("img")["src"]
            hidden_list = songs_list_wrapper_hidden.find_all("li") if songs_list_wrapper_hidden else []
            artist_songs = song_list_wrapper[0].find_all("li")
            all_songs = artist_songs+hidden_list
            results = []
            for song in all_songs:
                item = {}
                item["download_url"] = song.find("a",class_ = "btn btn-download")["href"]
                item["artist"] = song.find("span", class_="song-author").text
                item["genre"] = song.find("div",class_= "song-genre").text.split("/")[0].replace("#","").strip()
                item["title"] = song.find("span",class_="song-title").text
                item["image"] = artist_image
                item["source"] = song.find("a",class_ = "btn btn-play stream_play")["alt"]
                item["id"] = item["download_url"].split("/")[-1]
                results.append(item)
            return {"result":results,"count":len(results)}
        return []



from . import mdundo_url
import requests
from bs4 import BeautifulSoup

class Chart:

    def __init__(self,country):
        if country and len(country) != 2:
            raise ValueError("Invalid Country Abbreviation")
        self.url =  f"{mdundo_url}/best/{country}" if country else f"{mdundo_url}/best"

    def weekly_top_100(self):
        res = requests.get(self.url)
        soup = BeautifulSoup(res.text, 'lxml')
        songs_wrapper = soup.find_all("div",{"class":"song-list_wrapper"})
        weekly_top_100 = songs_wrapper[0].find_all("li")
        results = []
        for song in weekly_top_100:
            item = {}
            item["download_url"] = song.find("a",class_ = "btn btn-download")["href"]
            item["artist"] = song.find("span", class_="song-author").text.split(".",1)[1].strip()
            item["genre"] = song.find("div",class_= "song-genre").text.split("/")[0].replace("#","").strip()
            item["title"] = song.find("span",class_="song-title").text
            item["image"] = "https://picsum.photos/200"
            item["source"] = song.find("a",class_ = "btn btn-play stream_play")["alt"]
            item["id"] = item["download_url"].split("/")[-1]
            results.append(item)
        
        return {"result":results,"count":len(results)}

    def monthly_top_20(self):
        res = requests.get(self.url)
        soup = BeautifulSoup(res.text, 'lxml')
        songs_wrapper = soup.find_all("div",{"class":"song-list_wrapper"})
        monthly_top = songs_wrapper[1].find_all("li")
        results = []
        for song in monthly_top:
            item = {}
            item["download_url"] = song.find("a",class_ = "btn btn-download")["href"]
            item["artist"] = song.find("span", class_="song-author").text.split(".",1)[1].strip()
            item["genre"] = song.find("div",class_= "song-genre").text.split("/")[0].replace("#","").strip()
            item["title"] = song.find("span",class_="song-title").text
            item["id"] = item["download_url"].split("/")[-1]
            item["source"] = song.find("a",class_ = "btn btn-play stream_play")["alt"]
            item["image"] = "https://picsum.photos/200"
            results.append(item)
        return {"result":results,"count":len(results)}

    def new_releases(self):
        res = requests.get(self.url)
        soup = BeautifulSoup(res.text, 'lxml')
        songs_wrapper = soup.find_all("div",{"class":"song-list_wrapper"})
        new_releses_song = songs_wrapper[2].find_all("li")
        results = []
        for song in new_releses_song:
            item = {}
            item["download_url"] = song.find("a",class_ = "btn btn-download")["href"]
            item["artist"] = song.find("span", class_="song-author").text
            item["genre"] = song.find("div",class_= "song-genre").text.split("/")[0].replace("#","").strip()
            item["title"] = song.find("span",class_="song-title").text
            item["id"] = item["download_url"].split("/")[-1]
            item["source"] = song.find("a",class_ = "btn btn-play stream_play")["alt"]
            item["image"] = "https://picsum.photos/200"
            results.append(item)
        return {"result":results,"count":len(results)}


import setuptools
from setuptools import Extension

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='mdundo-scraper',
    description='Get songs data in json format from mdundo.com site with ease inspired by https://soundcloud-scraper.js.org/ ',
    long_description=long_description,
    long_description_content_type='text/markdown',
    version='0.0.5',
    url='https://github.com/Gidraf/mdundo-scraper',
    license='BSD-2',
    author='Gidraf Orenja',
    author_email='orenjagidraf@gmail.com',
    maintainer='Gidraf Orenjaa',
    maintainer_email='orenjagidraf@gmail.com',
    install_requires=[
        'beautifulsoup4',
        'requests-cache',
        'lxml',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development :: Libraries :: Python Modules',
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    project_urls={
        'Funding': 'https://donate.pypi.org',
        'Source': 'https://github.com/Gidraf/mdundo-scraper',
        'Tracker': 'https://github.com/Gidraf/mdundo-scraper/issues',
    },
    python_requires='>=3.6',
    zip_safe=False
)